@@ -1,4 +1,3 @@
-/* $Id$ */
 /*
  *  Copyright (c) 2008 Stephan Arts <stephan@xfce.org>
  *  Copyright (c) 2008 Jannis Pohlmann <jannis@xfce.org>
@@ -13,9 +12,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +27,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -53,6 +55,7 @@ enum
     COLUMN_THEME_NAME,
     COLUMN_THEME_DISPLAY_NAME,
     COLUMN_THEME_COMMENT,
+    COLUMN_THEME_NO_CACHE,
     N_THEME_COLUMNS
 };
 
@@ -79,6 +82,11 @@ static const gchar* xft_rgba_array[] =
     "none", "rgb", "bgr", "vrgb", "vbgr"
 };
 
+static const GtkTargetEntry theme_drop_targets[] =
+{
+  { "text/uri-list", 0, 0 }
+};
+
 /* Option entries */
 static GdkNativeWindow opt_socket_id = 0;
 static gboolean opt_version = FALSE;
@@ -284,6 +292,10 @@ appearance_settings_load_icon_themes (GtkListStore *list_store,
     gchar        *active_theme_name;
     gint          i;
     GSList       *check_list = NULL;
+    gchar        *cache_filename;
+    gboolean      has_cache;
+    gchar        *tmp;
+    gchar        *cache_tooltip;
 
     /* Determine current theme */
     active_theme_name = xfconf_channel_get_string (xsettings_channel, "/Net/IconThemeName", "Rodent");
@@ -332,12 +344,39 @@ appearance_settings_load_icon_themes (GtkListStore *list_store,
                     /* Escape the comment, since tooltips are markup, not text */
                     comment_escaped = theme_comment ? g_markup_escape_text (theme_comment, -1) : NULL;
 
+                    /* Cache filename */
+                    cache_filename = g_build_filename (icon_theme_dirs[i], file, "icon-theme.cache", NULL);
+                    has_cache = g_file_test (cache_filename, G_FILE_TEST_IS_REGULAR);
+                    g_free (cache_filename);
+
+                    /* If the theme has no cache, mention this in the tooltip */
+                    if (!has_cache)
+                      {
+                        cache_tooltip = g_strdup_printf (_("Warning: this icon theme has no cache file. You can create this by "
+                                                           "running <i>gtk-update-icon-cache %s/%s/</i> in a terminal emulator."),
+                                                         icon_theme_dirs[i], file);
+
+                        if (comment_escaped == NULL)
+                          {
+                            comment_escaped = cache_tooltip;
+                          }
+                        else
+                          {
+                            tmp = g_strconcat (comment_escaped, "\n\n", cache_tooltip, NULL);
+                            g_free (comment_escaped);
+                            g_free (cache_tooltip);
+                            comment_escaped = tmp;
+                          }
+                      }
+
                     /* Append icon theme to the list store */
                     gtk_list_store_append (list_store, &iter);
                     gtk_list_store_set (list_store, &iter,
                                         COLUMN_THEME_NAME, file,
                                         COLUMN_THEME_DISPLAY_NAME, theme_name,
-                                        COLUMN_THEME_COMMENT, comment_escaped, -1);
+                                        COLUMN_THEME_NO_CACHE, !has_cache,
+                                        COLUMN_THEME_COMMENT, comment_escaped,
+                                        -1);
 
                     /* Cleanup */
                     g_free (comment_escaped);
@@ -669,19 +708,132 @@ appearance_settings_dialog_channel_property_changed (XfconfChannel *channel,
     }
 }
 
+static void
+cb_theme_uri_dropped (GtkWidget        *widget,
+                      GdkDragContext   *drag_context,
+                      gint              x,
+                      gint              y,
+                      GtkSelectionData *data,
+                      guint             info,
+                      guint             timestamp,
+                      GtkBuilder       *builder)
+{
+    gchar        **uris;
+    gchar         *argv[3];
+    guint          i;
+    GError        *error = NULL;
+    gint           status;
+    GtkWidget     *toplevel = gtk_widget_get_toplevel (widget);
+    gchar         *filename;
+    GdkCursor     *cursor;
+    GdkWindow     *gdkwindow;
+    gboolean       something_installed = FALSE;
+    GObject       *object;
+    GtkTreeModel  *model;
+
+    uris = gtk_selection_data_get_uris (data);
+    if (uris == NULL)
+        return;
+
+    argv[0] = HELPERDIR G_DIR_SEPARATOR_S "appearance-install-theme";
+    argv[2] = NULL;
+
+    /* inform the user we are installing the theme */
+    gdkwindow = gtk_widget_get_window (widget);
+    cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget), GDK_WATCH);
+    gdk_window_set_cursor (gdkwindow, cursor);
+
+    /* iterate main loop to show cursor */
+    while (gtk_events_pending ())
+        gtk_main_iteration ();
+
+    for (i = 0; uris[i] != NULL; i++)
+    {
+        filename = g_filename_from_uri (uris[i], NULL, NULL);
+        if (filename == NULL)
+            continue;
+
+        argv[1] = filename;
+
+        if (g_spawn_sync (NULL, argv, NULL, 0, NULL, NULL, NULL, NULL, &status, &error)
+            && status > 0)
+        {
+            switch (WEXITSTATUS (status))
+            {
+                case 2:
+                    g_set_error (&error, G_SPAWN_ERROR, 0,
+                        _("File is larger than %d MB, installation aborted"), 50);
+                    break;
+
+                case 3:
+                    g_set_error_literal (&error, G_SPAWN_ERROR, 0,
+                        _("Failed to create temporary directory"));
+                    break;
+
+                case 4:
+                    g_set_error_literal (&error, G_SPAWN_ERROR, 0,
+                        _("Failed to extract archive"));
+                    break;
+
+                case 5:
+                    g_set_error_literal (&error, G_SPAWN_ERROR, 0,
+                        _("Unknown format, only archives and directories are supported"));
+                    break;
+
+                default:
+                    g_set_error (&error, G_SPAWN_ERROR,
+                        0, _("An unknown error, exit code is %d"), WEXITSTATUS (status));
+                    break;
+            }
+        }
+
+        if (error != NULL)
+        {
+            xfce_dialog_show_error (GTK_WINDOW (toplevel), error, _("Failed to install theme"));
+            g_clear_error (&error);
+        }
+        else
+        {
+            something_installed = TRUE;
+        }
+
+        g_free (filename);
+    }
+
+    g_strfreev (uris);
+    gdk_window_set_cursor (gdkwindow, NULL);
+    gdk_cursor_unref (cursor);
+
+    if (something_installed)
+    {
+        /* reload icon theme treeview */
+        object = gtk_builder_get_object (builder, "icon_theme_treeview");
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (object));
+        gtk_list_store_clear (GTK_LIST_STORE (model));
+        appearance_settings_load_icon_themes (GTK_LIST_STORE (model), GTK_TREE_VIEW (object));
+
+        /* reload gtk theme treeview */
+        object = gtk_builder_get_object (builder, "gtk_theme_treeview");
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (object));
+        gtk_list_store_clear (GTK_LIST_STORE (model));
+        appearance_settings_load_ui_themes (GTK_LIST_STORE (model), GTK_TREE_VIEW (object));
+    }
+}
+
 static void
 appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
 {
-    GObject          *object, *object2;
-    GtkListStore     *list_store;
-    GtkCellRenderer  *renderer;
-    GdkPixbuf        *pixbuf;
-    GtkTreeSelection *selection;
+    GObject           *object, *object2;
+    GtkListStore      *list_store;
+    GtkCellRenderer   *renderer;
+    GdkPixbuf         *pixbuf;
+    GtkTreeSelection  *selection;
+    GtkTreeViewColumn *column;
 
     /* Icon themes list */
     object = gtk_builder_get_object (builder, "icon_theme_treeview");
 
-    list_store = gtk_list_store_new (N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+    list_store = gtk_list_store_new (N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_store), COLUMN_THEME_DISPLAY_NAME, GTK_SORT_ASCENDING);
     gtk_tree_view_set_model (GTK_TREE_VIEW (object), GTK_TREE_MODEL (list_store));
     gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (object), COLUMN_THEME_COMMENT);
@@ -689,6 +841,13 @@ appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
     renderer = gtk_cell_renderer_text_new ();
     gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (object), 0, "", renderer, "text", COLUMN_THEME_DISPLAY_NAME, NULL);
 
+    renderer = gtk_cell_renderer_pixbuf_new ();
+    g_object_set (G_OBJECT (renderer), "icon-name", GTK_STOCK_DIALOG_WARNING, NULL);
+
+    column = gtk_tree_view_get_column (GTK_TREE_VIEW (object), 0);
+    gtk_tree_view_column_pack_start (column, renderer, FALSE);
+    gtk_tree_view_column_add_attribute (column, renderer, "visible", COLUMN_THEME_NO_CACHE);
+
     appearance_settings_load_icon_themes (list_store, GTK_TREE_VIEW (object));
 
     g_object_unref (G_OBJECT (list_store));
@@ -697,10 +856,15 @@ appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
     g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (cb_icon_theme_tree_selection_changed), NULL);
 
+    gtk_drag_dest_set (GTK_WIDGET (object), GTK_DEST_DEFAULT_ALL,
+                       theme_drop_targets, G_N_ELEMENTS (theme_drop_targets),
+                       GDK_ACTION_COPY);
+    g_signal_connect (G_OBJECT (object), "drag-data-received", G_CALLBACK (cb_theme_uri_dropped), builder);
+
     /* Gtk (UI) themes */
     object = gtk_builder_get_object (builder, "gtk_theme_treeview");
 
-    list_store = gtk_list_store_new (N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+    list_store = gtk_list_store_new (N_THEME_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_store), COLUMN_THEME_DISPLAY_NAME, GTK_SORT_ASCENDING);
     gtk_tree_view_set_model (GTK_TREE_VIEW (object), GTK_TREE_MODEL (list_store));
     gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (object), COLUMN_THEME_COMMENT);
@@ -716,6 +880,11 @@ appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
     g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (cb_ui_theme_tree_selection_changed), NULL);
 
+    gtk_drag_dest_set (GTK_WIDGET (object), GTK_DEST_DEFAULT_ALL,
+                       theme_drop_targets, G_N_ELEMENTS (theme_drop_targets),
+                       GDK_ACTION_COPY);
+    g_signal_connect (G_OBJECT (object), "drag-data-received", G_CALLBACK (cb_theme_uri_dropped), builder);
+
     /* Subpixel (rgba) hinting Combo */
     object = gtk_builder_get_object (builder, "xft_rgba_store");
 
@@ -806,8 +975,18 @@ appearance_settings_dialog_configure_widgets (GtkBuilder *builder)
 #endif
 }
 
+static void
+appearance_settings_dialog_response (GtkWidget *dialog,
+                                     gint       response_id)
+{
+    if (response_id == GTK_RESPONSE_HELP)
+        xfce_dialog_show_help (GTK_WINDOW (dialog), "xfce4-settings", "appearance", NULL);
+    else
+        gtk_main_quit ();
+}
+
 gint
-main(gint argc, gchar **argv)
+main (gint argc, gchar **argv)
 {
     GObject    *dialog, *plug_child;
     GtkWidget  *plug;
@@ -884,15 +1063,14 @@ main(gint argc, gchar **argv)
                 /* build the dialog */
                 dialog = gtk_builder_get_object (builder, "dialog");
 
-                gtk_widget_show (GTK_WIDGET (dialog));
-                g_signal_connect (dialog, "response", G_CALLBACK (gtk_main_quit), NULL);
+                g_signal_connect (dialog, "response",
+                    G_CALLBACK (appearance_settings_dialog_response), NULL);
+                gtk_window_present (GTK_WINDOW (dialog));
 
                 /* To prevent the settings dialog to be saved in the session */
                 gdk_set_sm_client_id ("FAKE ID");
 
                 gtk_main ();
-
-                gtk_widget_destroy (GTK_WIDGET (dialog));
             }
             else
             {
